@@ -22,9 +22,11 @@
 
 #include "config.h"
 
-#include "eggsmclient.h"
 #include "ephy-debug.h"
+#include "ephy-embed-prefs.h"
 #include "ephy-file-helpers.h"
+#include "ephy-private.h"
+#include "ephy-profile-utils.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
@@ -244,6 +246,7 @@ main (int argc,
   EphyStartupFlags startup_flags;
   EphyEmbedShellMode mode;
   int status;
+  EphyFileHelpersFlags flags;
 
 #ifdef ENABLE_NLS
   /* Initialize the i18n stuff */
@@ -333,7 +336,6 @@ main (int argc,
 #endif
 
   g_option_context_add_group (option_context, gtk_get_option_group (TRUE));
-  g_option_context_add_group (option_context, egg_sm_client_get_option_group ());
 
 #ifdef GNOME_ENABLE_DEBUG
   option_group = g_option_group_new ("debug",
@@ -387,10 +389,25 @@ main (int argc,
   /* Work-around Flash Player crash */
   g_setenv ("XLIB_SKIP_ARGB_VISUALS", "1", FALSE);
 
+  /* Run the migration in all cases, except when running a private
+     instance without a given profile directory. */
+  if (!(private_instance && profile_directory == FALSE)) {
+    /* If the migration fails we don't really want to continue. */
+    if (!ephy_profile_utils_do_migration ((const char *)profile_directory, -1, FALSE)) {
+      g_print ("Failed to run the migrator process, Web will now abort.");
+      exit (1);
+    }
+  }
+
   /* Start our services */
-  if (!ephy_file_helpers_init (profile_directory,
-                               private_instance || application_mode,
-                               keep_temp_directory || profile_directory,
+  flags = EPHY_FILE_HELPERS_ENSURE_EXISTS;
+
+  if (private_instance || application_mode)
+    flags |= EPHY_FILE_HELPERS_PRIVATE_PROFILE;
+  if (keep_temp_directory || profile_directory)
+    flags |= EPHY_FILE_HELPERS_KEEP_DIR;
+
+  if (!ephy_file_helpers_init (profile_directory, flags,
                                &error)) {
     show_error_message (&error);
     exit (1);
@@ -400,7 +417,7 @@ main (int argc,
                                           EPHY_PREFS_LOCKDOWN_ARBITRARY_URL);
 
   if (arguments != NULL && arbitrary_url) {
-    g_print ("URL loading is locked dow n\n");
+    g_print ("URL loading is locked down.\n");
     exit (1);
   }
 
@@ -431,8 +448,6 @@ main (int argc,
     exit (0);
   }
 
-  ephy_file_load_accels ();
-
   /* Now create the shell */
   if (private_instance)
     mode = EPHY_EMBED_SHELL_MODE_PRIVATE;
@@ -469,6 +484,7 @@ main (int argc,
     gtk_window_set_default_icon_name ("web-browser");
   }
 
+  ephy_embed_prefs_init ();
   _ephy_shell_create_instance (mode);
 
   startup_flags = get_startup_flags ();
@@ -488,8 +504,8 @@ main (int argc,
   if (notify_is_initted ())
     notify_uninit ();
 
-  ephy_file_save_accels ();
   ephy_state_save ();
+  ephy_embed_prefs_shutdown ();
   ephy_settings_shutdown ();
   ephy_file_helpers_shutdown ();
   xmlCleanupParser ();
